from AutoRPE.UtilsRPE.CodeTools import expand_sources
from AutoRPE.UtilsRPE.Functions import remove_comments
import re


def lines_from_filename(filename):
    with open(filename, "r", encoding="latin-1") as f:
        # Read, strip and keep only lines with content
        _lines = [_l.strip() for _l in f if _l.strip()]
    return _lines


def save_lines_into_file(_lines, filename):
    to_write = "\n".join(_lines).strip()
    with open(filename, "w") as f:
        f.write(to_write)


# Merge lines
# Definition:  given a file, puts back on the same line commands split with a &
def merge_lines(_lines):
    for _index, _line in enumerate(_lines):

        line = _lines[_index]
        if not line:
            continue
        if line[0] == "#":
            continue
        if line.count("&"):
            i = 1
            while line and line.strip()[-1] == "&":
                line1 = line
                line2 = _lines[_index + i]
                line = line1.replace("&", "").rstrip() + line2
                i += 1
            line = line.replace("&", "")
            # A commands to delete unnecessary spaces (gives some problems right now)
            #line = re.sub(" +", " ", line)
            _lines[_index] = line
            for j in range(1, i):
                _lines[_index + j] = ""
    return _lines


def remove_empty_lines(_lines):
    # Remove empty lines
    return [_l for _l in _lines if _l.strip()]


if __name__ == "__main__":
    from optparse import OptionParser
    parser = OptionParser()

    parser.add_option("-s", "--input-source", dest="input_source",
                      default=None, metavar="FILE")

    (options, args) = parser.parse_args()

    path = options.input_source

    if path is None:
        parser.print_help()
        exit(0)

    files = expand_sources(path)

    for index, fn in enumerate(files):
        # Read file
        lines = lines_from_filename(fn)
        # Remove comments ignoring preprocessor statements
        for line_index, line in enumerate(lines):
            if line[0] == "#":
                continue
            lines[line_index] = remove_comments(line)
        # Remove empty lines
        lines = remove_empty_lines(lines)

        # Merge lines
        lines = merge_lines(lines)
        # Remove empty lines again
        lines = remove_empty_lines(lines)
        save_lines_into_file(lines, fn)

